Package: jagsUI
Version: 1.6.3
Date: 2026-01-09
Title: A Wrapper Around 'rjags' to Streamline 'JAGS' Analyses
Authors@R: c(
    person("Ken", "Kellner", email="contact@kenkellner.com", role=c("cre","aut")),
    person("Mike", "Meredith", role="ctb")    
    )
Depends: R (>= 2.14.0),
Imports: coda (>= 0.13), graphics, grDevices, parallel, rjags (>=
        4-17), stats, utils
Suggests: knitr, markdown, tinytest
SystemRequirements: JAGS (http://mcmc-jags.sourceforge.net)
Description: A set of wrappers around 'rjags' functions to run Bayesian analyses in 'JAGS' (specifically, via 'libjags').  A single function call can control adaptive, burn-in, and sampling MCMC phases, with MCMC chains run in sequence or in parallel. Posterior distributions are automatically summarized (with the ability to exclude some monitored nodes if desired) and functions are available to generate figures based on the posteriors (e.g., predictive check plots, traceplots). Function inputs, argument syntax, and output format are nearly identical to the 'R2WinBUGS'/'R2OpenBUGS' packages to allow easy switching between MCMC samplers.
License: GPL-3
URL: https://kenkellner.com/jagsUI/
BugReports: https://github.com/kenkellner/jagsUI/issues
NeedsCompilation: no
VignetteBuilder: knitr
Packaged: 2026-01-08 17:33:43 UTC; ken
Author: Ken Kellner [cre, aut],
  Mike Meredith [ctb]
Maintainer: Ken Kellner <contact@kenkellner.com>
Repository: CRAN
Date/Publication: 2026-01-10 08:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 07:15:14 UTC; unix
