export(autojags)
export(densityplot)
export(jags)
export(jagsUI)
export(jags.basic)
export(jags.View)
export(pp.check)
export(traceplot)
export(whiskerplot)

S3method(plot, "jagsUI")
S3method(print,"jagsUI")
S3method(summary,"jagsUI")
S3method(update,"jagsUI")
S3method(update,"jagsUIbasic")
